/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.tx.interceptor;

import java.lang.reflect.Method;

import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.aop.spi.Invocation;
import org.t2framework.lucy.annotation.core.Inject;
import org.t2framework.lucy.tx.TransactionProcesser;
import org.t2framework.lucy.tx.Worker;

/**
 * 
 * {@.en Base transaction interceptor.}
 * 
 * <br />
 * 
 * {@.ja }
 * 
 * @author shot
 * 
 */
public abstract class AbstractTxInterceptor implements Interceptor {

	protected TransactionProcesser processer;

	protected WorkerAdapterFactory factory;

	public TransactionProcesser getProcesser() {
		return processer;
	}

	@Inject
	public void setProcesser(TransactionProcesser processer) {
		this.processer = processer;
	}

	public WorkerAdapterFactory getFactory() {
		return factory;
	}

	@Inject
	public void setFactory(WorkerAdapterFactory factory) {
		this.factory = factory;
	}

	@Override
	public Object intercept(Invocation<Method> invocation) throws Throwable {
		try {
			return process(getFactory().adapt(invocation));
		} catch (Throwable t) {
			throw t;
		}
	}

	protected abstract Object process(Worker<Object> worker) throws Throwable;
}
